# $Date: 1996/12/05 09:58:25 $ $Author: zimmerma $ $Revision: 1.42.2.3 $ #
#--
intlib::int(f,x) - computes the integral of the function f(x) with respect to 
the variable x.
--#
intlib::int:=proc(_F_,_x_)
option remember;
local opt,n,k_sub,k_ident,k_diff,k_deg,k_alg,Q,Q_sub,i,integral,tosubs,hassincos,k,hasI,Float,_f_;
begin
# test of input and options #
  if testargs()
    then 
         if domtype(_F_)<>DOM_IDENT and domtype(_F_)<>DOM_EXPR
            and not testtype(_F_,NUMERIC)
           then error("Illegal integrand") end_if;
  end_if;
  _F_:=subs(_F_,_x_=_x_); # to put _F_ in normal form in case it is not #
# First, transform each floating point number to its rational approximation #
  _f_:= maprec( _F_, {DOM_FLOAT,DOM_COMPLEX} = sharelib::rational);
  Float:=(if _f_ <> _F_ then float else id end_if);
  hassincos:=has(_f_,{sin,cos,tan}); hasI:=has(_f_,I);
  # lookup method #
  integral:=intlib::lookup(subs(_f_,_x_=_X));
  if integral<>FAIL then return( Float(subs(integral,_X=_x_)) ) end_if;
  opt:=hold(Exp);
  n:=1; k:=[_x_];
  k_sub:=[_x_]; k_ident:=[genident()]; k_diff:=[1]; k_deg:=[0]; k_alg:=[0];
  Q:=[]; Q_sub:=[];
# substitution and construction of the field extensions of the rational numbers#
  _f_:=intlib::subst(_f_);
  userinfo(2,"after substitution, integrand is",_f_);
  repeat i:=intlib::tower(_f_) until i<>FALSE end_repeat;
  if i=FAIL then
     userinfo(1,"construction of tower extension failed");
     return(Float(intlib::lastChance(args())))
  end_if;
  userinfo(2,"field extension of rational numbers",Q,Q_sub);
  userinfo(2,"tower extension of functions",k,k_sub,k_alg);
  userinfo(2,"after field extensions, integrand is",i);
  integral:=intlib::intt(i,n);
  if integral=FAIL then
     # try to simplify the integrand #
     assume(_x_,Type::RealNum);
     i:=simplify(args(1));
     if i<>_f_ then
        i:=rationalize(i,fun(domtype(args(1))=DOM_EXPR and type(args(1))<>"sign"));
        if i[2]<>{} then # to avoid loops #
           n:=intlib::int(i[1],_x_);
           if type(n)<>"int" then 
		return( Float(eval(subs(n,i[2]))) )
	   end_if
        end_if;
     end_if;
     return(Float(intlib::lastChance(args())))
  end_if;
  tosubs:=[k_sub[n-i]$hold(i)=0..n-2, Q_sub[nops(Q)-i]$hold(i)=0..nops(Q)-1];
  while has(integral,tosubs) do
     # take care with eval here, for example in int(1/(x^2+k^2),x) !!! #
     integral:=eval(subs(integral,k_sub[n-i]=k[n-i]$hold(i)=0..n-2,
                       Q_sub[nops(Q)-i]=Q[nops(Q)-i]$hold(i)=0..nops(Q)-1,
		hold(k)=hold(_intlib_k),hold(n)=hold(_intlib_n),
		hold(i)=hold(_intlib_i)))
  end_while;
  integral:=subs(integral,hold(_intlib_k)=hold(k),hold(_intlib_n)=hold(n),
	hold(_intlib_i)=hold(i));
  userinfo(2,"after substitution, integral is",integral);
  if hasI=FALSE and has(integral,I) then # gets only the real part #
     n:=sign(_x_);
     sysassign(sign(_x_),1);
     i:=rectform(integral,{_x_});
     if op(i,3)=0 then # unresolved part #
	i:=op(i,1); # real part #
	if not has(i,{Re,Im,abs}) then integral:=i end_if;
     end_if;
     sysassign(sign(_x_),(if type(n)<>"sign" then n end_if)); # else NIL #
  end_if;
  if hassincos then
     intlib::lookupsin; # to load intlib::to_sincos #
     integral:=combine(expand(intlib::to_sincos(integral,_x_)),sincos)
  end_if;
  Float(integral)
end_proc:

# if both the lookup method and Risch algorithm failed,
  and if the function is a sum, it can be that some
  terms can be integrated by the lookup method and some
  other by Risch algorithm, like for example int(diff(g(x),x)+x+2,x) #
intlib::lastChance := proc(f,x)
begin
   if type(f)="_mult" and contains(map({op(f)},type),"_plus") then
      f:=expand(f)
   end_if;
   if type(f)="_plus" then
      f:=map(f,intlib::int,x);
      if not has(f,int) then return(f) end_if
   end_if;
   hold(int)(args())
end_proc:
