# ode::isLODE returns TRUE if eq is an ordinary linear differential equation
    over the rational functions 
    option: "Lode" returns FALSE or eq expressed like (eq, y, x, n) 
Examples:
>> eq:=x^6*diff(y(x),x$6)+20*x^5*diff(y(x),x$5)+122*x^4*diff(y(x),x$4)+
   258*x^3*diff(y(x),x$3)+156*x^2*diff(y(x),x,x)+8*x*diff(y(x),x)-8*y(x):
>> ode::isLODE(eq, y(x));

                                   TRUE

>> ode::isLODE(eq, y(x), "Lode");
y(x)*(-8) + x*diff(y(x), x)*8 + x^2*diff(y(x), x, x)*156 + x^3*diff(\
y(x), x, x, x)*258 + x^4*diff(y(x), x, x, x, x)*122 + x^5*diff(y(x), x, x,\
 x, x, x)*20 + x^6*diff(y(x), x, x, x, x, x, x), y, x, 6
# 

ode::isLODE := proc(eq, z, o) 
  local vars, x, n, k;
  begin
    x := op(z); 
    vars := indets(eq, RatExpr) minus indets(eq);
    n := max(-infinity, op(map(vars, nops)))-1;
    case n
      of -infinity do if vars <> {} then return( FALSE ) end_if; break; 
      otherwise
        {diff(z, x$k) $k=0..n};
        if (vars minus % <> {}) or (degree(poly(eq, [op(vars)])) <> 1)
          then return( FALSE ) end_if
    end_case;
    if o="Lode" then eq, op(z, 0), x, n else TRUE end_if
   end_proc:
